﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Activities;
using Microsoft.TeamFoundation.Build.Client;

namespace Benday.TeamBuild2010.Activities
{
    [BuildActivity(HostEnvironmentOption.Agent)]
    public sealed class IisCreateWebApplication : CodeActivity
    {
        private IisUtility m_iisUtilityInstance;
        public IisUtility IisUtilityInstance
        {
            get
            {
                if (m_iisUtilityInstance == null)
                {
                    m_iisUtilityInstance = new IisUtility();
                }

                return m_iisUtilityInstance;
            }
        }

        [RequiredArgument]
        public InArgument<string> SiteName { get; set; }

        [RequiredArgument]
        public InArgument<string> AppPoolName { get; set; }

        [RequiredArgument]
        public InArgument<string> ApplicationName { get; set; }

        [RequiredArgument]
        public InArgument<string> ApplicationPath { get; set; }

        [RequiredArgument]
        public InArgument<bool> SetAuthenticationSettings { get; set; }

        [RequiredArgument]
        public InArgument<bool> AuthenticationAnonymous { get; set; }

        [RequiredArgument]
        public InArgument<bool> AuthenticationWindows { get; set; }

        [RequiredArgument]
        public InArgument<bool> AuthenticationBasic { get; set; }

        private string VerifySiteName(string siteName)
        {
            if (String.IsNullOrEmpty(siteName) == true)
            {
                var sites = IisUtilityInstance.GetSites();

                if (sites == null || sites.Count < 1)
                {
                    throw new InvalidOperationException("Could not determine default site name because IIS does not contain any sites.");
                }
                else
                {
                    return sites[0];
                }
            }
            else
            {
                return siteName;
            }
        }

        private void UpdateSecuritySettings(CodeActivityContext context, string siteName, string applicationName)
        {
            bool doSetAuthenticationSettings = SetAuthenticationSettings.Get(context);

            if (doSetAuthenticationSettings == false)
            {
                LogUtility.WriteToLog(context,
                    "IisCreateWebApplication: Skipped update to security settings.",
                    BuildMessageImportance.Low);
            }
            else
            {
                LogUtility.WriteToLog(context,
                    "IisCreateWebApplication: Setting security settings for web application.",
                    BuildMessageImportance.Low);

                var allowAnonymous = AuthenticationAnonymous.Get(context);
                var allowWindows = AuthenticationWindows.Get(context);
                var allowBasic = AuthenticationBasic.Get(context);

                LogUtility.WriteToLog(context,
                    String.Format("IisCreateWebApplication: Anonymous = '{0}'.", allowAnonymous),
                    BuildMessageImportance.Low);

                LogUtility.WriteToLog(context,
                    String.Format("IisCreateWebApplication: Basic = '{0}'.", allowBasic),
                    BuildMessageImportance.Low);

                LogUtility.WriteToLog(context,
                    String.Format("IisCreateWebApplication: Windows = '{0}'.", allowWindows),
                    BuildMessageImportance.Low);

                SecuritySettings settings = new SecuritySettings() 
                    { Anonymous = allowAnonymous, Basic = allowBasic, Windows = allowWindows };

                IisUtilityInstance.SetSecurityForApplication(siteName, applicationName, settings);

                LogUtility.WriteToLog(context,
                    "IisCreateWebApplication: Security settings for web application set.",
                    BuildMessageImportance.Low);
            }
        }

        private void UpdateAppPoolSettings(CodeActivityContext context, string tempAppPoolName, string tempSiteName, string tempApplicationName)
        {
            if (String.IsNullOrEmpty(tempAppPoolName) == false)
            {
                LogUtility.WriteToLog(context,
                "IisCreateWebApplication: Setting app pool for web application.",
                BuildMessageImportance.Low);

                IisUtilityInstance.SetAppPoolForApplication(tempSiteName,
                    tempApplicationName,
                    tempAppPoolName);

                LogUtility.WriteToLog(context,
                "IisCreateWebApplication: App pool for web application set.",
                BuildMessageImportance.Low);
            }
            else
            {
                LogUtility.WriteToLog(context,
                    "IisCreateWebApplication: Skipped setting app pool.",
                    BuildMessageImportance.Low);
            }
        }

        protected override void Execute(CodeActivityContext context)
        {
            try
            {
                LogUtility.WriteToLog(context,
                    "IisCreateWebApplication: Preparing to create web application.",
                    BuildMessageImportance.Low);

                string tempAppPoolName = AppPoolName.Get(context);
                string tempSiteName = SiteName.Get(context);
                string tempApplicationName = ApplicationName.Get(context);
                string tempApplicationPath = ApplicationPath.Get(context);

                LogUtility.WriteToLog(context,
                    String.Format("IisCreateWebApplication: (AppPoolName: '{0}', SiteName: '{1}', ApplicationName: '{2}', ApplicationPath: '{3}')", 
                    tempAppPoolName, tempSiteName, tempApplicationName, tempApplicationPath),
                    BuildMessageImportance.Low);

                tempSiteName = VerifySiteName(tempSiteName);

                IisUtilityInstance.CreateApplication(tempSiteName, tempApplicationName, tempApplicationPath);

                UpdateAppPoolSettings(context, tempAppPoolName, tempSiteName, tempApplicationName);
                UpdateSecuritySettings(context, tempSiteName, tempApplicationName);

                LogUtility.WriteToLog(context,
                        String.Format("IisCreateWebApplication created web application -- AppPoolName: '{0}', SiteName: '{1}', ApplicationName: '{2}', ApplicationPath: '{3}'",
                        tempAppPoolName, tempSiteName, tempApplicationName, tempApplicationPath),
                        BuildMessageImportance.High);
            }
            catch (Exception ex)
            {
                LogUtility.WriteToLog(context, String.Format("IisCreateWebApplication: An exception occurred.  {0}",
                                                   ex.ToString()), BuildMessageImportance.High);
            }
        }
    }
}
